using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using SharedNamespace;

public partial class _Default : System.Web.UI.Page
{
    internal MapService_MapServer MapServiceProxy
    {
        get
        {
            object obj = Session["mapServiceProxy"];
            if (obj == null)
            {
                MapService_MapServer mapService = new MapService_MapServer();
                mapService.Url = "https://sampleserver5.arcgisonline.com/arcgis/services/SF311/MapServer";
                obj = mapService;
                Session["mapServiceProxy"] = obj;
            }
            return obj as MapService_MapServer;
        }
    }

    internal GeocodeService_GeocodeServer GeocodeServiceProxy
    {
        get
        {
            object obj = Session["geocodeServiceProxy"];
            if (obj == null)
            {
                GeocodeService_GeocodeServer geocodeService = new GeocodeService_GeocodeServer();
                geocodeService.Url = "http://sampleserver1.arcgisonline.com/ArcGIS/services/Locators/ESRI_Geocode_USA/GeocodeServer";
                obj = geocodeService;
                Session["geocodeServiceProxy"] = obj;
            }
            return obj as GeocodeService_GeocodeServer;
        }
    }

    internal Geometry_GeometryServer GeometryServiceProxy
    {
        get
        {
            object obj = Session["geometryServiceProxy"];
            if (obj == null)
            {
                Geometry_GeometryServer geometryService = new Geometry_GeometryServer();
                geometryService.Url = "http://sampleserver1.arcgisonline.com/arcgis/services/Geometry/GeometryServer";
                obj = geometryService;
                Session["geometryServiceProxy"] = obj;
            }
            return obj as Geometry_GeometryServer;
        }
    }

    // Register button to initiate partial postback, geocode, project, redraw map
    protected void Page_Load(object sender, EventArgs e)
    {
        ScriptManager1.RegisterAsyncPostBackControl(Button1);
    }

    // Initial map draw
    protected void Page_PreRender(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            MapServerInfo mapInfo = MapServiceProxy.GetServerInfo(MapServiceProxy.GetDefaultMapName());
            MapDescription mapDesc = mapInfo.DefaultMapDescription;
            DrawMap(mapDesc);
        }
    }

    protected void DrawMap(MapDescription mapDesc)
    {
        ImageType imgType = new ImageType();
        imgType.ImageFormat = esriImageFormat.esriImagePNG;
        imgType.ImageReturnType = esriImageReturnType.esriImageReturnMimeData;

        ImageDisplay imgDisp = new ImageDisplay();
        imgDisp.ImageHeight = (int)Image1.Height.Value;
        imgDisp.ImageWidth = (int)Image1.Width.Value;
        imgDisp.ImageDPI = 96;

        ImageDescription imgDesc = new ImageDescription();
        imgDesc.ImageDisplay = imgDisp;
        imgDesc.ImageType = imgType;

        MapImage mapImage = MapServiceProxy.ExportMapImage(mapDesc, imgDesc);

        string imgKey = Guid.NewGuid().ToString();

        Image1.ImageUrl = string.Format("ImageHandler.ashx?imgKey={0}", imgKey);

        HttpContext.Current.Cache.Add(imgKey, mapImage.ImageData, null, DateTime.MaxValue,
            TimeSpan.FromSeconds(5), System.Web.Caching.CacheItemPriority.Default, null);
    }

    // Button click event to initiate process
    protected void Button1_Click(object sender, EventArgs e)
    {
        MapServerInfo mapInfo = MapServiceProxy.GetServerInfo(MapServiceProxy.GetDefaultMapName());
        MapDescription mapDesc = mapInfo.DefaultMapDescription;
        MapLayerInfo[] mapLayerInfos = mapInfo.MapLayerInfos;

        ImageDisplay imgDisp = new ImageDisplay();
        imgDisp.ImageHeight = (int)Image1.Height.Value;
        imgDisp.ImageWidth = (int)Image1.Width.Value;
        imgDisp.ImageDPI = 96;

        double scale = MapServiceProxy.ComputeScale(mapDesc, imgDisp);

        // Geocode address
        PointN mapGeocodePoint = GeocodeAddress(TextBox1.Text, TextBox2.Text, mapDesc.SpatialReference);
        
        // Buffer geometry
        Geometry[] geocodePointBuffers = BufferPoint(mapGeocodePoint, mapDesc);

        // Bounding extent of the buffers
        EnvelopeN boundingExtent = GetBoundingExtent(geocodePointBuffers);

        // Recenter map
        CenterAndSize centerSize = new CenterAndSize();
        centerSize.Center = mapGeocodePoint;
        centerSize.Height = Math.Abs(boundingExtent.YMax - boundingExtent.YMin) * 1.2;
        centerSize.Width = Math.Abs(boundingExtent.XMax - boundingExtent.XMin) * 1.2;
        mapDesc.MapArea = centerSize;

        // Add geocode point and buffer polygon as graphics
        RgbColor geocodePointRgbColor = new RgbColor();
        geocodePointRgbColor.Red = 0;
        geocodePointRgbColor.Green = 255;
        geocodePointRgbColor.Blue = 0;

        RgbColor geocodePointOutlineRgbColor = new RgbColor();
        geocodePointOutlineRgbColor.Red = 0;
        geocodePointOutlineRgbColor.Green = 0;
        geocodePointOutlineRgbColor.Blue = 0;

        SimpleMarkerSymbol geocodePointMarkerSymbol = new SimpleMarkerSymbol();
        geocodePointMarkerSymbol.Style = esriSimpleMarkerStyle.esriSMSCircle;
        geocodePointMarkerSymbol.Color = geocodePointRgbColor;
        geocodePointMarkerSymbol.Size = 10.0;

        geocodePointMarkerSymbol.Outline = true;
        geocodePointMarkerSymbol.OutlineColor = geocodePointOutlineRgbColor;
        geocodePointMarkerSymbol.OutlineSize = 1;

        MarkerElement geocodeMarker = new MarkerElement();
        geocodeMarker.Symbol = geocodePointMarkerSymbol;
        geocodeMarker.Point = mapGeocodePoint;

        RgbColor bufferRgbColor = new RgbColor();
        bufferRgbColor.Red = 255;
        bufferRgbColor.Green = 0;
        bufferRgbColor.Blue = 0;

        SimpleFillSymbol bufferFillSymbol = new SimpleFillSymbol();
        SimpleLineSymbol bufferOutlineLineSymbol = new SimpleLineSymbol();
        bufferOutlineLineSymbol.Color = bufferRgbColor;
        bufferOutlineLineSymbol.Style = esriSimpleLineStyle.esriSLSSolid;
        bufferOutlineLineSymbol.Width = 2;

        bufferFillSymbol.Outline = bufferOutlineLineSymbol;
        bufferFillSymbol.Style = esriSimpleFillStyle.esriSFSForwardDiagonal;
        bufferFillSymbol.Color = bufferRgbColor;

        PolygonElement bufferPolygonElement = new PolygonElement();
        bufferPolygonElement.Symbol = bufferFillSymbol;
        bufferPolygonElement.Polygon = (PolygonN)geocodePointBuffers[0];

        AddGraphics(mapDesc, bufferPolygonElement);
        AddGraphics(mapDesc, geocodeMarker);

        // Use buffer to query features in map service
        MapLayerInfo queryMapLayerInfo = null;
        foreach (MapLayerInfo layerinfo in mapLayerInfos)
        {
            if (layerinfo.Name == "Incidents")
            {
                queryMapLayerInfo = layerinfo;
            }
        }
        QueryData(mapDesc, queryMapLayerInfo, (PolygonN)geocodePointBuffers[0]);

        // Draw map
        DrawMap(mapDesc);

    }

    protected PointN GeocodeAddress(string street, string zone, SpatialReference mapSR)
    {
        // Define address inputs
        PropertySetProperty geocodePropAddress = new PropertySetProperty();
        geocodePropAddress.Key = "Address";
        geocodePropAddress.Value = street; // "600 Montgomery Street";

        PropertySetProperty geocodePropCity = new PropertySetProperty();
        geocodePropCity.Key = "City";
        geocodePropCity.Value = ""; // "San Francisco";

        PropertySetProperty geocodePropState = new PropertySetProperty();
        geocodePropState.Key = "State";
        geocodePropState.Value = ""; // "CA";

        PropertySetProperty geocodePropZip = new PropertySetProperty();
        geocodePropZip.Key = "Zip";
        geocodePropZip.Value = zone; // "94111";

        PropertySet geocodePropSet = new PropertySet();
        geocodePropSet.PropertyArray = new PropertySetProperty[] { geocodePropAddress, geocodePropCity, geocodePropState, geocodePropZip };

        PropertySet locatorPropSet = GeocodeServiceProxy.GetLocatorProperties();

        PropertySetProperty[] newPropertySet = new PropertySetProperty[locatorPropSet.PropertyArray.Length + 1];
        for (int i = 0; i < locatorPropSet.PropertyArray.Length; i++)
        {
            newPropertySet[i] = locatorPropSet.PropertyArray[i];
        }

        PropertySetProperty outputSpatialReferenceProperty = new PropertySetProperty();
        outputSpatialReferenceProperty.Key = "OutputSpatialReference";
        outputSpatialReferenceProperty.Value = mapSR;

        newPropertySet[locatorPropSet.PropertyArray.Length] = outputSpatialReferenceProperty;
        locatorPropSet.PropertyArray = newPropertySet;

        // Geocode address
        PropertySet geocodeResults = GeocodeServiceProxy.GeocodeAddress(geocodePropSet, locatorPropSet);
        PropertySetProperty[] resultsArray = geocodeResults.PropertyArray;

        PointN geocodePoint = null;
        foreach (PropertySetProperty geocodeResult in resultsArray)
        {
            if (geocodeResult.Value is PointN)
            {
                geocodePoint = geocodeResult.Value as PointN;
                break;
            }
        }

        return geocodePoint;
    }

    protected Geometry[] BufferPoint(PointN inputPoint, MapDescription mapInfo)
    {
        SpatialReference operationalSpatialReference = CreateOperationSpatialReference(inputPoint);

        Geometry[] outputGeometry = GeometryServiceProxy.Project(inputPoint.SpatialReference, operationalSpatialReference, true, null, null, new Geometry[] { inputPoint });

        PointN projectedPoint = (PointN)outputGeometry[0];
        double searchWidth = 50;
        double searchHeight = 30;

        // Create a cross with geocode point as center
        Path yPath = new Path();
        PointN pnt1 = new PointN();
        pnt1.X = projectedPoint.X - searchWidth;
        pnt1.Y = projectedPoint.Y;
        PointN pnt2 = new PointN();
        pnt2.X = projectedPoint.X + searchWidth;
        pnt2.Y = projectedPoint.Y;
        yPath.PointArray = new PointN[] { pnt1, pnt2 };

        Path xPath = new Path();
        PointN pnt12 = new PointN();
        pnt12.X = projectedPoint.X;
        pnt12.Y = projectedPoint.Y - searchHeight;
        PointN pnt22 = new PointN();
        pnt22.X = projectedPoint.X;
        pnt22.Y = projectedPoint.Y + searchHeight;
        xPath.PointArray = new PointN[] { pnt12, pnt22 };

        PolylineN crossPolyline = new PolylineN();
        crossPolyline.PathArray = new Path[] { yPath, xPath };

        // Specify buffer distances
        double[] bufferDistances = new double[] { 300 };

        // Linear unit in feet (http://edndoc.esri.com/arcobjects/9.2/ComponentHelp/esrigeometry/esriSRUnitType.htm)
        LinearUnit linearUnit = new LinearUnit();
        linearUnit.WKID = 9003;
        linearUnit.WKIDSpecified = true;

        Geometry[] geometryArrayOutput = GeometryServiceProxy.Buffer(operationalSpatialReference, operationalSpatialReference, mapInfo.SpatialReference, bufferDistances, linearUnit, false, new Geometry[] { crossPolyline });

        return geometryArrayOutput;
    }

    // Gets the bounding extent of the passed-in ArcGIS Server SOAP geometries
    public EnvelopeN GetBoundingExtent(Geometry[] agsSoapGeometryArray)
    {
        // Instantiate an envelope with max values minimized and min values maximized
        EnvelopeN agsSoapBoundingBox = new EnvelopeN();
        agsSoapBoundingBox.XMin = double.MaxValue;
        agsSoapBoundingBox.XMax = double.MinValue;
        agsSoapBoundingBox.YMin = double.MaxValue;
        agsSoapBoundingBox.YMax = double.MinValue;

        for (int i = 0; i < agsSoapGeometryArray.Length; i++)
        {
            Geometry agsSoapGeometry = agsSoapGeometryArray[i];
            if (agsSoapGeometry is PolygonN)
            {
                PolygonN agsSoapPolygon = agsSoapGeometry as
                    PolygonN;

                agsSoapBoundingBox.SpatialReference = agsSoapPolygon.SpatialReference;

                // Iterate through all the polygon's vertices
                for (int j = 0; j < agsSoapPolygon.RingArray.Length; j++)
                {
                    Ring agsSoapRing = agsSoapPolygon.RingArray[j];
                    for (int k = 0; k < agsSoapRing.PointArray.Length; k++)
                    {
                        // For each vertex, expand the bounds of the minimum enclosing rectangle with the 
                        // vertex's coordinates if they fall outside the rectangle's current bounds
                        PointN agsSoapCurrentPoint = agsSoapRing.PointArray[k]
                            as PointN;

                        if (agsSoapCurrentPoint.X < agsSoapBoundingBox.XMin)
                            agsSoapBoundingBox.XMin = agsSoapCurrentPoint.X;
                        else if (agsSoapCurrentPoint.X > agsSoapBoundingBox.XMax)
                            agsSoapBoundingBox.XMax = agsSoapCurrentPoint.X;

                        if (agsSoapCurrentPoint.Y < agsSoapBoundingBox.YMin)
                            agsSoapBoundingBox.YMin = agsSoapCurrentPoint.Y;
                        else if (agsSoapCurrentPoint.Y > agsSoapBoundingBox.YMax)
                            agsSoapBoundingBox.YMax = agsSoapCurrentPoint.Y;
                    }
                }
            }
            else if (agsSoapGeometry is PolylineN)
            {
                PolylineN agsSoapPolyline = agsSoapGeometry as
                    PolylineN;

                agsSoapBoundingBox.SpatialReference = agsSoapPolyline.SpatialReference;

                // Iterate through all the polyline's vertices
                for (int j = 0; j < agsSoapPolyline.PathArray.Length; j++)
                {
                    Path agsSoapPath = agsSoapPolyline.PathArray[j];
                    for (int k = 0; k < agsSoapPath.PointArray.Length; k++)
                    {
                        // For each vertex, expand the bounds of the minimum enclosing rectangle with the 
                        // vertex's coordinates if they fall outside the rectangle's current bounds
                        PointN agsSoapCurrentPoint = agsSoapPath.PointArray[k]
                            as PointN;

                        if (agsSoapCurrentPoint.X < agsSoapBoundingBox.XMin)
                            agsSoapBoundingBox.XMin = agsSoapCurrentPoint.X;
                        else if (agsSoapCurrentPoint.X > agsSoapBoundingBox.XMax)
                            agsSoapBoundingBox.XMax = agsSoapCurrentPoint.X;

                        if (agsSoapCurrentPoint.Y < agsSoapBoundingBox.YMin)
                            agsSoapBoundingBox.YMin = agsSoapCurrentPoint.Y;
                        else if (agsSoapCurrentPoint.Y > agsSoapBoundingBox.YMax)
                            agsSoapBoundingBox.YMax = agsSoapCurrentPoint.Y;
                    }
                }
            }
            else if (agsSoapGeometry is PointN)
            {
                PointN agsSoapPoint = agsSoapGeometry
                    as PointN;

                agsSoapBoundingBox.SpatialReference = agsSoapPoint.SpatialReference;

                agsSoapBoundingBox.XMin = agsSoapPoint.X - .1;
                agsSoapBoundingBox.XMax = agsSoapPoint.X + .1;
                agsSoapBoundingBox.YMin = agsSoapPoint.Y - .1;
                agsSoapBoundingBox.YMax = agsSoapPoint.Y + .1;
            }
        }

        return agsSoapBoundingBox;
    }

    public SpatialReference CreateOperationSpatialReference(Geometry agsSoapGeometry)
    {
        // Get the polygon's minimum enclosing rectangle (MER)
        EnvelopeN agsSoapBoundingEnvelope = GetBoundingExtent(new Geometry[] { agsSoapGeometry });

        // If the input polygon's spatial reference uses a projected coordinate system, project the MER to a 
        // geographic coordinate system (WGS 1984 in this case).  We do this because the MER's coordinates 
        // will be used to initialize the datum of the operation spatial reference

        if (agsSoapBoundingEnvelope.SpatialReference is ProjectedCoordinateSystem)
        {
            // Create an ArcGIS Server spatial reference initalized to use the WGS 1984 coordinate system
            SpatialReference agsSoapGeographicSpatialReference = new GeographicCoordinateSystem();
            agsSoapGeographicSpatialReference.WKID = 4326;
            agsSoapGeographicSpatialReference.WKIDSpecified = true;

            // Place the input MER in an array for the project operation
            Geometry[] agsSoapInputGeometryArray = new Geometry[1];
            agsSoapInputGeometryArray[0] = agsSoapBoundingEnvelope;

            // Execute the projection
            Geometry[] agsSoapOutputGeometryArray =
                GeometryServiceProxy.Project(agsSoapBoundingEnvelope.SpatialReference, agsSoapGeographicSpatialReference,
                true, null, null, agsSoapInputGeometryArray);

            // Retrieve the projected MER from the results array
            agsSoapBoundingEnvelope = agsSoapOutputGeometryArray[0] as EnvelopeN;
        }

        // Get the latitude (Y coordinate) at the center of the MER
        double centerLatitude = agsSoapBoundingEnvelope.YMax -
            (agsSoapBoundingEnvelope.YMax - agsSoapBoundingEnvelope.YMin) / 2;

        // Create the definition string for the operation spatial reference's coordinate system.  We will use 
        // the Hotine Oblique Mercator coordinate system because it lends itself well to minimizing operational
        // distortion anywhere on the earth
        string hotineObliqueMercatorDefinition = @"
            PROJCS[""World_Hotine"",
            GEOGCS[""GCS_WGS_1984"",
            DATUM[""D_WGS_1984"",
            SPHEROID[""WGS_1984"",6378137.0,298.257223563]],
            PRIMEM[""Greenwich"",0.0],
            UNIT[""Degree"",0.0174532925199433]],
            PROJECTION[""Hotine_Oblique_Mercator_Two_Point_Natural_Origin""],
            PARAMETER[""False_Easting"",0.0],
            PARAMETER[""False_Northing"",0.0],
            PARAMETER[""Latitude_Of_1st_Point"",{0}],   
            PARAMETER[""Latitude_Of_2nd_Point"",{1}],
            PARAMETER[""Scale_Factor"",1.0],
            PARAMETER[""Longitude_Of_1st_Point"",{2}],
            PARAMETER[""Longitude_Of_2nd_Point"",{3}],
            PARAMETER[""Latitude_Of_Center"",{4}],
            UNIT[""Meter"",1.0]]";

        // Specify the relevant coordinates of the MER for the coordinate system's datum parameters
        string customHotineObliqueCylindricalMercator = string.Format(
            hotineObliqueMercatorDefinition, agsSoapBoundingEnvelope.YMin,
            agsSoapBoundingEnvelope.YMax, agsSoapBoundingEnvelope.XMin,
            agsSoapBoundingEnvelope.XMax, centerLatitude);

        // Create the spatial reference
        SpatialReference agsSoapMeasureSpatialReference =
            GeometryServiceProxy.FindSRByWKT(customHotineObliqueCylindricalMercator, null, true, true);

        return agsSoapMeasureSpatialReference;
    }

    protected MapDescription AddGraphics(MapDescription mapDesc, GraphicElement graphicElement)
    {
        if (mapDesc.CustomGraphics != null)
        {
            GraphicElement[] originalElements = mapDesc.CustomGraphics;
            int elementCount = originalElements.Length;
            GraphicElement[] newElements = new GraphicElement[elementCount + 1];
            originalElements.CopyTo(newElements, 0);
            newElements[elementCount] = graphicElement;
            mapDesc.CustomGraphics = newElements;
        }
        else
        {
            GraphicElement[] elements = new GraphicElement[1];
            elements[0] = graphicElement;
            mapDesc.CustomGraphics = elements;
        }

        return mapDesc;
    }

    private void QueryData(MapDescription mapDesc, MapLayerInfo layerInfo, PolygonN bufferPolygon)
    {
        MapService_MapServer mapService = MapServiceProxy;

        string geometryFieldName = string.Empty;
        Field[] fields = layerInfo.Fields.FieldArray;
        foreach (Field field in fields)
        {
            if (field.Type == esriFieldType.esriFieldTypeGeometry)
            {
                geometryFieldName = field.Name;
                break;
            }
        }

        SpatialFilter spatialFilter = new SpatialFilter();
        spatialFilter.GeometryFieldName = geometryFieldName;
        spatialFilter.SpatialRel = esriSpatialRelEnum.esriSpatialRelIntersects;
        spatialFilter.FilterGeometry = bufferPolygon;
        RecordSet recordSet = null;
        FIDSet fidset = null;

        try
        {
            recordSet = mapService.QueryFeatureData(mapDesc.Name, layerInfo.LayerID, spatialFilter);
            fidset = mapService.QueryFeatureIDs(mapDesc.Name, layerInfo.LayerID, spatialFilter);
        }
        catch (Exception ex)
        {
            // Improper format of where clause will cause exception
        }

        RgbColor querySelectionColor = new RgbColor();
        querySelectionColor.Red = 255;
        querySelectionColor.Green = 255;
        querySelectionColor.Blue = 0;

        SimpleMarkerSymbol queryMarkerSymbol = new SimpleMarkerSymbol();
        queryMarkerSymbol.Style = esriSimpleMarkerStyle.esriSMSDiamond;
        queryMarkerSymbol.Color = querySelectionColor;
        queryMarkerSymbol.Size = 18.0;

        RgbColor queryRgbOutline = new RgbColor();
        queryRgbOutline.Red = 0;
        queryRgbOutline.Green = 0;
        queryRgbOutline.Blue = 0;

        queryMarkerSymbol.Outline = true;
        queryMarkerSymbol.OutlineColor = queryRgbOutline;
        queryMarkerSymbol.OutlineSize = 2;

        TextSymbol textSymbol = new TextSymbol();
        textSymbol.Color = queryRgbOutline;
        textSymbol.Size = 10;
        textSymbol.FontName = "Arial";
        textSymbol.XOffset = -2;
        textSymbol.YOffset = 6;


        DataSet dataSet = new DataSet();
        DataTable dataTable = new DataTable();

        if (recordSet != null)
        {

            // Include these fields
            System.Collections.ArrayList fieldList =
                new System.Collections.ArrayList() { "req_id", "req_type", "req_date", "req_time", "address", "status" };

            foreach (Field field in recordSet.Fields.FieldArray)
            {                
                if (fieldList.Contains(field.Name))
                {
                    DataColumn dc = new DataColumn(field.AliasName, typeof(string));
                    dataTable.Columns.Add(dc);
                }
            }

            // Iterate through results
            foreach (Record record in recordSet.Records)
            {
                DataRow dataRow = dataTable.NewRow();
                string resultValues = string.Empty;
                object[] values = record.Values;

                string idFieldValue = null;
                PointN pointMarker = null;

                int v = 0;
                foreach (Field field in recordSet.Fields.FieldArray)
                {
                    if (field.Type != esriFieldType.esriFieldTypeGeometry)
                    {
                        // If datatable contains a matching field, set value
                        if (dataTable.Columns.Contains(field.AliasName))
                        {
                            if (values[v] != null)
                            {
                                if (field.Name == "status")
                                {
                                    if (values[v].ToString() == "1")
                                    {
                                        dataRow[field.AliasName] = "New";
                                    }
                                    else if (values[v].ToString() == "2")
                                    {
                                        dataRow[field.AliasName] = "Open";
                                    }
                                    else if (values[v].ToString() == "3")
                                    {
                                        dataRow[field.AliasName] = "Closed";
                                    }
                                }
                                else
                                {
                                    dataRow[field.AliasName] = values[v].ToString();
                                }
                            }
                            else
                            {
                                dataRow[field.AliasName] = "";
                            }
                        }

                        if (field.Name == "req_id")
                            idFieldValue = values[v].ToString();

                       // if (field.Type == esriFieldType.esriFieldTypeOID)
                            //idFieldValue = values[v].ToString();

                    }
                    else
                    {
                        pointMarker = values[v] as PointN;
                    }
                    v++;
                }

                MarkerElement resultsMarker = new MarkerElement();
                resultsMarker.Symbol = queryMarkerSymbol;
                resultsMarker.Point = pointMarker;
                AddGraphics(mapDesc, resultsMarker);

                TextElement te = new TextElement();
                te.Symbol = textSymbol;
                te.Text = idFieldValue;
                te.TextGeometry = pointMarker;
                AddGraphics(mapDesc, te);

                dataTable.Rows.Add(dataRow);
            }
        }

        // Change column names, if desired
        //dataTable.Columns["VisitTime"].ColumnName = "Wait Time";

        dataSet.Tables.Add(dataTable);
        GridView1.DataSource = dataSet;

        GridView1.DataBind();
        GridView1.Visible = true;
        GridView1.BorderWidth = 10;
    }    
}
